#ifndef FRAMEBUFFER_OBJECT
#define FRAMEBUFFER_OBJECT

#include "gpuDLL.h"
#include <glew.h>
#include <vector> 

namespace gpu {

  class GPU_API FramebufferObject {
  public:
    FramebufferObject();
    ~FramebufferObject();
  
    inline void bind();
    inline static void unbind();

    void attachTexture(GLenum tex_target, 
                       GLuint tex_id,
                       GLenum attachment = GL_COLOR_ATTACHMENT0_EXT,
                       int mip_level     = 0,
                       int z_slice       = 0);

    void attachRenderBuffer(GLuint rb_id,
                            GLenum attachment = GL_COLOR_ATTACHMENT0_EXT);
  
    void unattach(GLenum attachment);
    void unattachAll();

    bool isValid();

    GLenum getAttachedType(GLenum attachment);
    GLuint getAttachedId(GLenum attachment);
    GLint  getAttachedMipLevel(GLenum attachment);
    GLint  getAttachedCubeFace(GLenum attachment);
    GLint  getAttachedZSlice(GLenum attachment);
  
    static int getMaxColorAttachments();
    static GLenum *buffers(unsigned int i=0);

  private:
    void  unbindCurrentBindThis();
    void  unbindThisBindCurrent();

    GLuint fbo_id;
    GLint  current_fbo_id; 
  };

  inline void FramebufferObject::bind() {
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,fbo_id);
  }

  inline void FramebufferObject::unbind() {
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
  }
 
} // gpu namespace

#endif //  FRAMEBUFFER_OBJECT
